﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualBasic.FileIO;
using System.Xml.Serialization;
using System.IO;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Drawing;

namespace DesktopStation
{
    class TrackData
    {
        public int No;
        public Int16 ImageNo;
        public Int16 AccsessoryAddress;
        public Int16 S88SensorAddress;
        public int RouteNo;

        public TrackData()
        {
            No = 0;
            ImageNo = 0;
            AccsessoryAddress = 0;
            S88SensorAddress = 0;
            RouteNo = -1;

        }

    }

    class TrackDiagramManager
    {
        public List<TrackData> Items;
        public int Width;
        public int Height;
        public bool UsingS88;
        private Bitmap RailMapImage;

        public TrackDiagramManager()
        {
            Items = new List<TrackData>();

            Width = Program.TILE32_LAYOUT_WIDTH;
            Height = Program.TILE32_LAYOUT_HEIGHT;
            UsingS88 = false;

        }

        public void Clear()
        {
            int i;
            
            Items.Clear();
            UsingS88 = false;

            for (i = 0; i < Width * Height; i++)
            {
                TrackData aItem = new TrackData();
                aItem.No = i;
                Items.Add(aItem);
            }           


        }

        public int GetLayoutTile(int inMapIndex)
        {
            if (inMapIndex < Items.Count)
            {
                return (Items[inMapIndex].ImageNo);
            }
            else
            {
                return 0;
            }

        }

        public int GetLayoutAccNo(int inMapIndex)
        {
            if (inMapIndex < Items.Count)
            {
                return (Items[inMapIndex].AccsessoryAddress);
            }
            else
            {
                return 0;
            }

        }

        public int GetLayoutS88No(int inMapIndex)
        {
            if (inMapIndex < Items.Count)
            {
                return (Items[inMapIndex].S88SensorAddress);
            }
            else
            {
                return 0;
            }

        }

        public int GetLayoutRouteNo(int inMapIndex)
        {
            if (inMapIndex < Items.Count)
            {
               return (Items[inMapIndex].RouteNo);

            }
            else
            {
                return 0;
            }

            }

        public void SetLayoutMapRouteNo(int inMapIndex, int inRouteNo)
        {
            if (inMapIndex < Items.Count)
            {
                Items[inMapIndex].RouteNo = inRouteNo;
            }

        }

        public void UpdateLayoutTile(int inMapIndex, int inTileNo)
        {

            /* 値を更新 */
            Items[inMapIndex].ImageNo = (Int16)inTileNo;

        }

        public void UpdateLayoutAccNo(int inMapIndex, int inAccAddr)
        {

            /* 値を更新 */
            Items[inMapIndex].AccsessoryAddress = (Int16)inAccAddr;

        }

        public void UpdateLayoutS88Addr(int inMapIndex, int inS88Addr)
        {
            /* フラグ書き換え */
            if (inS88Addr > 0)
            {
                UsingS88 = true;
            }

            /* 値を更新 */
            Items[inMapIndex].S88SensorAddress = (Int16)inS88Addr;

        }


        public int GetLayoutMapIndex(int inX, int inY, int inScaleRatio)
        {
            int aX;
            int aY;
            int aTileSize = Program.TILE32_SIZE * inScaleRatio / 100;

            /* XYのインデックス番号に変換 */
            aX = inX / aTileSize;
            aY = inY / aTileSize;

            /* 横方向の最大を規定 */
            if (aX >= Width)
            {

                aX = Width - 1;
            }

            /* 縦方向の最大を規定 */
            if (aY >= Height)
            {

                aY = Height - 1;
            }

            /* 1次元のインデックス番号を確定 */
            return (aY * Width + aX);

        }

        public void SaveToFile(String inFileName)
        {
            int i;

            //ファイルを作る
            StreamWriter aStrWriter = new StreamWriter(inFileName, false);

            for (i = 0; i < Items.Count; i++)
            {
                /* 書き込み */
                aStrWriter.WriteLine("{0},{1},{2},{3}", Items[i].ImageNo, Items[i].AccsessoryAddress, Items[i].S88SensorAddress, Items[i].RouteNo);
            }

            aStrWriter.Close();
        }

        public void LoadFromFile(String inFileName)
        {

            String[] aFields;
            int i;
            int aTileNo;
            int aAccAddr;
            int aS88Addr;
            int aRouteNo;
            int aMax;

            //設定ファイルがあるときのみ読み込みします。
            if (File.Exists(inFileName))
            {

                TextFieldParser aParser = new TextFieldParser(inFileName);
                aParser.TextFieldType = FieldType.Delimited;
                aParser.SetDelimiters(",");

                i = 0;
                aMax = Width * Height;

                Items.Clear();

                while (aParser.EndOfData == false)
                {

                    /* 分析処理する */
                    aFields = aParser.ReadFields();

                    aTileNo = GetCSVFieldInt(aFields, 0, 0);
                    aAccAddr = GetCSVFieldInt(aFields, 1, 0);
                    aS88Addr = GetCSVFieldInt(aFields, 2, 0);
                    aRouteNo = GetCSVFieldInt(aFields, 3, 0);

                    /* 追加 */
                    if (i < aMax)
                    {
                        TrackData aItem = new TrackData();

                        aItem.ImageNo = (Int16)aTileNo;
                        aItem.AccsessoryAddress = (Int16)aAccAddr;
                        aItem.S88SensorAddress = (Int16)aS88Addr;
                        aItem.RouteNo = aRouteNo;

                        if (aS88Addr > 0)
                        {
                            UsingS88 = true;
                        }

                        Items.Add(aItem);

                    }

                    /* インデックスを進める */
                    i = i + 1;

                }

                aParser.Close();

                /* サイズが満たない場合は空データを追加 */
                while (i < aMax)
                {
                    TrackData aItem2 = new TrackData();
                    Items.Add(aItem2);

                    i++;
                }


            }
            else
            {
                /* 存在しない場合は、空データだけ生成する */
                Clear();

            }

        }

        private static int GetCSVFieldInt(String[] inFields, int inIndex, int inDefault)
        {
            int aResult;

            if (inFields.Length > inIndex)
            {
                aResult = ParseStrToInt(inFields[inIndex]);
            }
            else
            {
                aResult = inDefault;
            }
            return aResult;
        }

        public static int ParseStrToInt(String aText)
        {
            int aRet;

            Int32.TryParse(aText, out aRet);

            return aRet;

        }


        public void LoadRailMap(String inFilename)
        {
            RailMapImage = new Bitmap(inFilename);

        }

        public void DrawRail(Graphics inCanvas, int inIndex, int x, int y, int width, int height)
        {
            if (RailMapImage == null)
            {
                return;
            }

            int ax, ay;

            ax = (inIndex % 4) * 32;
            ay = (inIndex / 4) * 32;


            Rectangle srcRect = new Rectangle(ax, ay, 32, 32);

            Rectangle desRect = new Rectangle(x, y, width, height);

            inCanvas.DrawImage(RailMapImage, desRect, srcRect, GraphicsUnit.Pixel);

        }


    }
}
